#!/bin/bash

# environment variables settings
RKDEVTOOL=${RKDEVTOOL:=rkdeveloptool}

cd $(dirname $0)

function finish() {
    read -rsp $'\nPress any key to finish\n' -n1 key
}

trap finish EXIT

# check if needed files exist
if [ ! -d files ]; then
    echo "Cannot find extra files directory."
    echo "Please redownload, extract tarball and run ./dwarf-factory-reset.sh again."
    exit 102
fi

if [ ! -f files/idbloader-rk3399.img ] || [ ! -f files/modduox-gr-rk3399-v1.9.1.66-restore.img ] || [ ! -f files/parameter_gpt.txt ] || [ ! -f files/rk3399_loader_v1.24.124.bin ]; then
    echo "Cannot find extra files needed for deployment."
    echo "Please redownload, extract tarball and run ./dwarf-factory-reset.sh again."
    exit 102
fi

cd files

if ! command -v ${RKDEVTOOL}; then
    if uname -s | grep -q Darwin; then
        RKDEVTOOL="./rkdeveloptool.macos"
        echo "Using local rkdeveloptool.macos"
    elif uname -m | grep -q aarch64; then
        RKDEVTOOL="./rkdeveloptool.linux-arm64"
        echo "Using local rkdeveloptool.linux-arm64"
    else
        RKDEVTOOL="./rkdeveloptool.linux-x86_64"
        echo "Using local rkdeveloptool.linux-x86_64"
    fi
else
    echo "Using ${RKDEVTOOL} as \$RKDEVTOOL"
fi

# check if MOD is in flashing mode
if command -v lsusb >/dev/null && ! lsusb -d "2207:330c"; then
    echo "MOD Dwarf is not connected or is not running in flashing mode."
    echo "Turn MOD off / on again while activating maskrom mode, with shorting the SoM emmc pin to ground as last resort."
    exit 101
fi

echo "Checking current CPU status..."

# case 1: CPU not initialized, so a forced maskrom via shorting emmc to ground
if ! ${RKDEVTOOL} td 1>/dev/null; then
    echo "Initializing CPU"
    if ! ${RKDEVTOOL} db rk3399_loader_v1.24.124.bin; then
        echo "Failed to initialize CPU"
        exit 201
    fi

    sleep 1

    if ! ${RKDEVTOOL} ul rk3399_loader_v1.24.124.bin; then
        echo "Failed to update CPU loader"
        exit 201
    fi

    sleep 1

    echo "Clearing all flash data, please wait..."
    ${RKDEVTOOL} ef

    sleep 1

    if ! ${RKDEVTOOL} gpt parameter_gpt.txt; then
        echo "Failed to write partition table"
        exit 201
    fi

# case 2: CPU initialized, triggered by uboot
elif ${RKDEVTOOL} ppt | grep -q system; then
    echo "Partition table already exists, reset everything, please wait..."
    ${RKDEVTOOL} ef
    sleep 1
    echo "Rebooting unit"
    ${RKDEVTOOL} rd
    sleep 2
    if ! ${RKDEVTOOL} td 1>/dev/null; then
        if ! ${RKDEVTOOL} db rk3399_loader_v1.24.124.bin; then
            echo "Failed to initialize CPU"
            exit 201
        fi
    fi
    sleep 2
    if ! ${RKDEVTOOL} ul rk3399_loader_v1.24.124.bin; then
        echo "Failed to update CPU loader after clearing flash data"
        exit 201
    fi
    if ! ${RKDEVTOOL} gpt parameter_gpt.txt; then
        echo "Failed to write partition table after clearing flash data"
        exit 201
    fi

# case 3: CPU initialized, triggered by CPU loader (failed to get into uboot)
else
    echo "Clearing all flash data and reset, please wait..."
    ${RKDEVTOOL} ef
    sleep 1
    echo "Rebooting unit"
    ${RKDEVTOOL} rd
    sleep 1
    if ! ${RKDEVTOOL} td 1>/dev/null; then
        if ! ${RKDEVTOOL} db rk3399_loader_v1.24.124.bin; then
            echo "Failed to initialize CPU"
            exit 201
        fi
        sleep 2
    fi
    if ! ${RKDEVTOOL} gpt parameter_gpt.txt; then
        echo "Failed to write partition table after clearing flash data"
        exit 201
    fi

fi

echo "Flashing CPU loader"
sleep 1
${RKDEVTOOL} wl 0x40 idbloader-rk3399.img

echo "Flashing restore"
sleep 1
${RKDEVTOOL} wl 0x2000 modduox-gr-rk3399-v1.9.1.66-restore.img

if [ $? -ne 0 ]; then
    echo "Factory deploy was interrupted. Please try again."
    exit 201
fi

echo "Rebooting into restore"
${RKDEVTOOL} rd

echo "Done"
exit 0
